--1) for each month, how many products have been purchased in the month by at
--least 30 distinct customers each?

with member nprods as 
count(filter([Product].[Product Id].[Product Id], [Measures].[NCustomers] >=20))
select nprods on columns,
nonempty(([Time].[The Month].[The Month])) on rows
from sales

--2) for each store, and considering only Saturdays, the products that have sold at
--least 1% of sales of the store;

with member totsales as 
([Product].[Product Id].currentmember.parent, [Measures].[Store Sales])
member percsales as [Measures].[Store Sales]/totsales
select {[Measures].[Store Sales], totsales} on columns,
nonempty( 
Filter( ([Store].[Store Id].[Store Id],[Product].[Product Id].[Product Id]), percsales>0.01) ) on rows
from sales
where [Time].[WeekDays].[The Day].&[Saturday]

-- 3) for every product category, percentage of customers buying that product category
--on Tuesday's over the total number of customers buying that category;

with member perc as  
([Time].[WeekDays].[The Day].&[Tuesday], [Measures].[NCustomers])  / [Measures].[NCustomers],
FORMAT_STRING="Percent" 
select perc on columns,
[Product].[Product Category].[Product Category] on rows 
from sales

-- 4) For every customer city, number of different product 
--categories bought by customers of that city
with member ncat as  
Count( nonempty( DISTINCT([Product].[Product Category].[Product Category], [Measures].[Store Sales]) ) ) 
select ncat on columns,
[Customer].[City].[City] on rows 
from sales 

-- 5) for every product category and customer country, the store with the highest 
-- percentage of sales over the total sales of that category and country. Result must be ordered by
-- percentage descending.

WITH member total as ([Store].[Store Id].[All], [Measures].[Store Sales]) 
MEMBER perc AS [Measures].[Store Sales]/total,
FORMAT_STRING="Percent" 
SELECT perc on columns, 
Order(
 {Generate( nonempty(([Product].[Product Category].[Product Category], [Customer].[Country].[Country])),
  topcount( ([Product].[Product Category].currentmember,[Customer].[Country].currentmember,
             [Store].[Store Id].[Store Id]), 1, perc))},
 perc, bdesc) on rows
FROM sales

-- Sol. Analytic SQL
with rk as 
(select pc.product_category, c.country, s.store_id, 
         sum(store_sales)/sum(sum(store_sales)) over (partition by pc.product_category, c.country) as ratio,
         rank() over (partition by pc.product_category, c.country order by sum(store_sales) desc) as rk
from sales_fact s join product p on s.product_id = p.product_id
       join product_class pc on p.product_class_id = pc.product_class_id
       join customer c on s.customer_id = c.customer_id 
group by pc.product_category, c.country, s.store_id) 

select product_category, country, store_id, ratio
from rk where rk=1
order by ratio desc

--6) for every product, the date with the highest sales of the product. If there are
--two or more days with the same highest sales, pick up any date.

select [Measures].[Store Sales]  on columns,
generate( [Product].[Product Id].[Product Id],
 topcount( ([Product].[Product Id].currentmember, [Time].[Time Id].[Time Id]), 
 1, [Measures].[Store Sales]) ) on rows
from sales

-- Analytic SQL
with temp as (
select product_id, time_id, sum(store_sales) as sales, 
rank() over (partition by product_id order by sum(store_sales), time_id desc) as rk
from sales_fact
group by product_id, time_id)

select product_id, time_id, sales
from temp
where rk=1

-- 20140205
with member supp as [Measures].[Sales Fact Count]
member suppA as ([Store].[Geography].[All], [Measures].[Sales Fact Count])
member conf as supp/suppA

select {supp, suppA, conf} on columns,
filter( (descendants([Product].[Marketing]),descendants([Store].[Geography])), conf>0.1) on rows
from sales

select [Measures].[Sales Fact Count] on columns,
descendants([Product].[Marketing] on rows
from sales
 

-- 4/4/2014 For the City of Losa Angeles, customer full name (No fulnn name, so Id) and total sales 
-- to the customer and ration of sales done during travel 
with member LA_sales as
([Store].[Sales City].&[Los Angeles], [Measures].[Store Sales])
member tot_sales as [Measures].[Store Sales]
member trav_sales as (tot_sales - LA_sales)/tot_sales
--,format_string="percent"

select {tot_sales, trav_sales} on columns,
nonempty([Customer].[Customer Id].[Customer Id]) on rows
from sales
where [Customer].[City].&[Los Angeles]

--14/4/2015 for each store,  the name and the total sales of the top-profit customers that sum up to 1000 dollars of profit.
select {[Measures].[Store Sales], [Measures].[Profit]} on columns,
generate (
[Store].[Geography].[Store Id], topsum(
crossjoin(
{[Store].[Geography].currentmember},[Customer].[Customer Id].[Customer Id]), 1000, [Measures].[Profit]
)
) on rows
from sales

--top	5	stores	in	March	1998 with the highest ratio	
--between	 the	store	sales	of	 females	and	
--store	sales	of	all	customers	in	the	whole	1998

with member den as
(([Customer].[Gender].[All], [Time].[DayMonthYear].currentmember.parent.parent), 
[Measures].[Store Sales])
member ratio as
case when den=0 then 0 else [Measures].[Store Sales]/den end
select{[Measures].[Store Sales], ratio, den} on columns,
topcount(nonempty(([Store].[Store Id].[Store Id])), 5, ratio) on rows
from sales
where ([Time].[DayMonthYear].[The Year].&[1998].&[Q1].&[March],[Customer].[Gender].&[F])


-- for each store and month return the difference between the 
--mean profit per customer for that month and the mean profit per customer for that year
with member year_sales as
([Time].[DayMonthYear].currentmember.parent, [MeanProfitPerCustomer])
member delta as [Measures].[MeanProfitPerCustomer]-year_sales
select {[Measures].[Store Sales],[Measures].[Profit],
[Measures].[MeanProfitPerCustomer], year_sales, delta} on columns,
nonempty(
([Store].[Store Id].[Store Id], [Time].[The Year].[The Year], 
[Time].[DayMonthYear].[The Month]
)
) on rows
from sales

-- creating the metric 
with 
member year_sales as
([Time].[DayMonthYear].currentmember.parent, [Measures].[Profit])/
([Time].[DayMonthYear].currentmember.parent, [Measures].[NCustomers])
member delta as 
([Measures].[Profit]/[Measures].[NCustomers])-year_sales
select {[Measures].[Store Sales],[Measures].[Profit],
[Measures].[MeanProfitPerCustomer], year_sales, delta, a} on columns,
nonempty(
([Store].[Store Id].[Store Id], [Time].[The Year].[The Year], 
[Time].[DayMonthYear].[The Month]
)
) on rows
from sales


--  For every product family the number of distinct female customers with 
-- a profit higher than 50000 who bought that product family 
select {[Measures].[NCustomers],[Measures].[Profit]} on columns,
Filter([Product].[Product Family].[Product Family],[Measures].[Profit]>50000) on rows 
--([Product].[Product Family].[Product Family]) on rows 
from sales 
where [Customer].[Gender].&[F]


-- For every product family with the ratioSex higher than 0.5 
-- ratioSex is given by the store sales of female customers over 
-- the store sales of male customers 
with member ratio as 
([Measures].[Store Sales])/([Customer].[Gender].&[M], [Measures].[Store Sales])
select {ratio} on columns,
Filter([Product].[Product Family].[Product Family],ratio>1.02) on rows 
from sales 
where [Customer].[Gender].&[F]



